%**************************************************************************
% State evolution for Fig. 2 in 
%   http://arxiv.org/pdf/1602.06509v1.pdf
%**************************************************************************
clear all;

rho = 0.1;    %

delta = 0.35;

SNRdB = 50;

Iteration = 25;

MSE_SE_AMP = zeros(Iteration,1);
MSE_SE_OAMP_LMMSE = zeros(Iteration,1);
MSE_SE_OAMP_MF = zeros(Iteration,1);
MSE_SE_OAMP_PINV = zeros(Iteration,1);

% psi function generated using Monte Carlo simulation
if(rho==0.1)
    load Table_sparsity01_real;
    SNR_Table = Table_sparsity01_real(:,1);     % in linear scale
    VAR_Table = Table_sparsity01_real(:,2); 
else
    fprintf('error\n');
    pause;
end

SNR_MIN = min(SNR_Table);
SNR_MAX = max(SNR_Table);

% State evolution prediction
sigma2 = 10^(-SNRdB/10) * 1/delta;

v2_AMP = 1;
v2_OAMP_MF = 1;
v2_OAMP_PINV = 1;
v2_OAMP_LMMSE = 1;
% AMP    
for it = 1:Iteration   
    
    % AMP
    tau2_AMP = 1/delta * v2_AMP + sigma2;
    
    eta_AMP = max( 1/tau2_AMP, SNR_MIN);
    eta_AMP = min(eta_AMP, SNR_MAX);

    v2_AMP = interp1q(SNR_Table,VAR_Table,eta_AMP); 
    MSE_SE_AMP(it) = v2_AMP;
    
    % OAMP-MF
    tau2_OAMP_MF = 1/delta * v2_OAMP_MF + sigma2;
    
    eta_OAMP_MF = max( 1/tau2_OAMP_MF, SNR_MIN);
    eta_OAMP_MF = min(eta_OAMP_MF, SNR_MAX);

    v2_post = interp1q(SNR_Table,VAR_Table,eta_OAMP_MF);
    v2_OAMP_MF = 1/(1/v2_post - eta_OAMP_MF );
    MSE_SE_OAMP_MF(it) = v2_post;
    
    % OAMP-PINV
    tau2_OAMP_PINV = (1/delta - 1) * v2_OAMP_PINV + sigma2 * 1/(1-delta);
    
    eta_OAMP_PINV = max( 1/tau2_OAMP_PINV, SNR_MIN);
    eta_OAMP_PINV = min(eta_OAMP_PINV, SNR_MAX);

    v2_post = interp1q(SNR_Table,VAR_Table,eta_OAMP_PINV);
    v2_OAMP_PINV = 1/(1/v2_post - eta_OAMP_PINV );
    MSE_SE_OAMP_PINV(it) = v2_post;
    
    % OAMP-LMMSE
    c = 1/delta - 1;
    tau2_OAMP_LMMSE = (sigma2 + c * v2_OAMP_LMMSE + sqrt( ( sigma2 + c * v2_OAMP_LMMSE )^2 + 4 * sigma2 * v2_OAMP_LMMSE ) ) / 2;
    
    eta_OAMP_LMMSE = max( 1/tau2_OAMP_LMMSE, SNR_MIN);
    eta_OAMP_LMMSE = min(eta_OAMP_LMMSE, SNR_MAX);

    v2_post = interp1q(SNR_Table,VAR_Table,eta_OAMP_LMMSE);
    v2_OAMP_LMMSE = 1/(1/v2_post - eta_OAMP_LMMSE );
    MSE_SE_OAMP_LMMSE(it) = v2_post;
end
figure

semilogy(1:Iteration,MSE_SE_OAMP_MF,'.',1:Iteration,MSE_SE_AMP,'.',...
        1:Iteration,MSE_SE_OAMP_PINV,'.',1:Iteration,MSE_SE_OAMP_LMMSE,'.');
    
legend('OAMP-MF','AMP','OAMP-PINV','OAMP-LMMSE');
xlabel('iteration');
ylabel('MSE');