% Generate the Table for SE analysis using Monte Carlo simulation
% This can also be computed via integration for Bernoulli-Gaussian prior

clear all;

rho = 0.1;    % 0.02 non-zeros

N = 1e6;

NSIM = 50;
SNRdB = [-20:1:80];

MSE = zeros(1,length(SNRdB));

for iii = 1:length(SNRdB)
    
    sigma2 = 10^(-SNRdB(iii)/10);
    
    rand('state',2);
    randn('state',2);

    for nsim=1:NSIM                                                       
        index = random('Binomial',1,rho,N,1);
        x = sqrt(1/rho) * randn(N,1);
        x = x .* index;        
        
        y = x + sqrt(sigma2)* randn(N,1);     

        exponent =  - abs(y).^2 * 1/rho / ( sigma2 * (sigma2+1/rho));
        exponent = min(exponent, 50);
        exponent = max(exponent, -50);

        exponent = exponent / 2;   
        den = 1 + sqrt( (sigma2+1/rho)/sigma2 ) * (1-rho)/rho * exp(exponent); 
        C = 1./den;
        x_hat = ( 1/rho ) / ( 1/rho + sigma2 ) * y .* C;
        Var = 1/rho*sigma2/(sigma2+1/rho) * C + abs(1/rho/(1/rho+sigma2) * y).^2 .* C .* (1-C);
        
        MSE(1,iii) = MSE(1,iii) + norm(x_hat - x)^2/(N);
        
        if(mod(nsim,5)==0)
            fprintf('**********************************************\n');
            fprintf('snr = %d dB, nsim = %d\n',SNRdB(iii),nsim);
            fprintf('MSE = %e\n',MSE(1,iii)/nsim);
        end        
    end

end

MSE = MSE/NSIM;

SNR = 10.^(SNRdB/10);
Table_sparsity01_real = [SNR',MSE'];
save('Table_sparsity01_real.mat','Table_sparsity01_real');

