%**************************************************************************
% State evolution for Fig. 3 in 
%   http://arxiv.org/pdf/1602.06509v1.pdf
%**************************************************************************
clear all;

rho = 0.2;    %

N = 4000;
M = 2000;
delta = M/N;

SNRdB = 60;

Iteration = 35;

MSE_SE_OAMP_LMMSE = zeros(Iteration,1);
MSE_SE_OAMP_MF = zeros(Iteration,1);
MSE_SE_OAMP_PINV = zeros(Iteration,1);

% psi function generated using Monte Carlo simulation
if(rho==0.2)
    load Table_sparsity02_real;
    SNR_Table = Table_sparsity02_real(:,1);     % in linear scale
    VAR_Table = Table_sparsity02_real(:,2); 
else
    fprintf('error\n');
    pause;
end

SNR_MIN = min(SNR_Table);
SNR_MAX = max(SNR_Table);

% State evolution prediction
sigma2 = 10^(-SNRdB/10) * 1/delta;

v2_OAMP_MF = 1;
v2_OAMP_PINV = 1;
v2_OAMP_LMMSE = 1;

%% set up the singular values, column normalization
kappa = 5;

d2 = kappa.^(-2/M*[0:M-1]);
d2 = d2/sum(d2) * N;
d = sqrt(d2)';
% AMP    
for it = 1:Iteration 
    
    % OAMP-MF
    tau2_OAMP_MF = ( sum(abs(d2).^2)/N * v2_OAMP_MF + sum(d2)/N * sigma2 ) / ( sum(d2)/N )^2 - v2_OAMP_MF;
    
    eta_OAMP_MF = max( 1/tau2_OAMP_MF, SNR_MIN);
    eta_OAMP_MF = min(eta_OAMP_MF, SNR_MAX);

    v2_post = interp1q(SNR_Table,VAR_Table,eta_OAMP_MF);
    v2_OAMP_MF = 1/(1/v2_post - eta_OAMP_MF );
    MSE_SE_OAMP_MF(it) = v2_post;
    
    % OAMP-PINV
    tau2_OAMP_PINV = (1/delta - 1) * v2_OAMP_PINV + 1/delta * sum( 1./d2 ) / M * sigma2;
    
    eta_OAMP_PINV = max( 1/tau2_OAMP_PINV, SNR_MIN);
    eta_OAMP_PINV = min(eta_OAMP_PINV, SNR_MAX);

    v2_post = interp1q(SNR_Table,VAR_Table,eta_OAMP_PINV);
    v2_OAMP_PINV = 1/(1/v2_post - eta_OAMP_PINV );
    MSE_SE_OAMP_PINV(it) = v2_post;
    
    % OAMP-LMMSE
    v_post = v2_OAMP_LMMSE - (v2_OAMP_LMMSE)^2 * sum( abs(d).^2 ./ ( v2_OAMP_LMMSE * abs(d).^2 + sigma2) ) / N;
    tau2_OAMP_LMMSE = 1/(1/v_post - 1/v2_OAMP_LMMSE);
      
    eta_OAMP_LMMSE = max( 1/tau2_OAMP_LMMSE, SNR_MIN);
    eta_OAMP_LMMSE = min(eta_OAMP_LMMSE, SNR_MAX);

    v2_post = interp1q(SNR_Table,VAR_Table,eta_OAMP_LMMSE);
    v2_OAMP_LMMSE = 1/(1/v2_post - eta_OAMP_LMMSE );
    MSE_SE_OAMP_LMMSE(it) = v2_post;
end
figure

semilogy(1:Iteration,MSE_SE_OAMP_MF,'.',...
        1:Iteration,MSE_SE_OAMP_PINV,'.',1:Iteration,MSE_SE_OAMP_LMMSE,'.');
    
legend('OAMP-MF','OAMP-PINV','OAMP-LMMSE');
xlabel('iteration');
ylabel('MSE');