%**************************************************************************
% Simulation for AMP for Fig. 4
% of http://arxiv.org/pdf/1602.06509v1.pdf
%**************************************************************************

clear all;

rho = 0.2;    % sparsity level

N = 500;     
M = fix(N * 0.5);  
delta = M/N;

% N = 4;     
% M = fix(N * 1);  
% delta = M/N;

NSIM = 500;
SNRdB = 60;

Iteration = 1000;

sigma2 = 1/delta * 10^(-SNRdB/10);    

Kappa = [1:1:1e1];    % condition number of the matrix
% Kappa = 5;
MSE_simulation = zeros(length(Kappa),1);

%% Simulation
for iii = 1:length(Kappa)
    
    %% set up the singular values, column normalization
    kappa = Kappa(iii);

    d2 = kappa.^(-2/M*[0:M-1]);
    d2 = d2/sum(d2) * N;
    d = sqrt(d2)';
    
    for nsim=1:NSIM                                                       
        index = random('Binomial',1,rho,N,1);
        x = sqrt(1/rho) * randn(N,1);
        x = x .* index;

        % column normalized matrix
        temp = randn(N,N);
        [V_full,~] = qr(temp);
        V = V_full(1:M,:);

        temp = randn(M,M);
        [U,~] = qr(temp);

        A = U * diag(d) * V; 

        y = A * x + sqrt(sigma2) * randn(M,1); 

        % AMP recovery
        z_hat = zeros(M,1);
        div = 0;
        x_hat = zeros(N,1);

        stop = 0;

        x_hat_old = zeros(N,1);

        it = 0;
        
        while (~stop) 

            it = it + 1;

            z_hat = y - A * x_hat + 1/delta * div * z_hat;
            r_hat = A' * z_hat + x_hat;

            tau2 = norm(z_hat)^2/M;

            % scalar MMSE estimator      
            exponent =  - abs(r_hat).^2 * 1/rho ./ ( tau2 .* (tau2+1/rho));
            exponent = min(exponent, 50);
            exponent = max(exponent, -50);
            exponent = exponent / 2; 
            den = 1 + sqrt( (tau2+1/rho)./tau2) .* (1-rho)/rho .* exp(exponent); 
            C = 1./den;

            % conditional mean and variance
            x_hat = ( 1/rho ) ./ ( 1/rho + tau2 ) .* r_hat .* C;
            Var = 1/rho*tau2./(tau2+1/rho) .* C + abs(1/rho./(1/rho+tau2) .* r_hat).^2 .* C .* (1-C);

            % divergence
            div = mean(Var)/tau2;

            if( norm(x_hat-x_hat_old)^2/norm(x_hat)^2 < 1e-10 || it > Iteration )
                stop = 1;
            end

            x_hat_old = x_hat;
            
        end   

        % measure MSE, use x_hat_post

        MSE_simulation(iii) = MSE_simulation(iii) + norm(x_hat - x)^2/N;

        if(mod(nsim,10)==0)
            fprintf('******************************************************\n');
            fprintf('kappa = %e, nsim = %d\n',kappa,nsim);
            fprintf('MSE = %e\n',MSE_simulation(iii)/nsim);
        end

    end
    
end

MSE_simulation = MSE_simulation/NSIM;

MSE_simulation = MSE_simulation/NSIM;

figure;
loglog(Kappa,MSE_simulation,'r-');
legend('AMP');
xlabel('Kappa');
ylabel('MSE');
